// =====================================================================================================================
//  fennec, a free and open source game engine
//  Copyright © 2025  Medusa Slockbower
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <https://www.gnu.org/licenses/>.
// =====================================================================================================================

///
/// \file scalar.h
/// \brief the \ref page_fennec_math_scalar
///
///
/// \details
/// \author Medusa Slockbower
///
/// \copyright Copyright © 2025 Medusa Slockbower ([GPLv3](https://www.gnu.org/licenses/gpl-3.0.en.html))
///
///

#ifndef FENNEC_MATH_SCALAR_H
#define FENNEC_MATH_SCALAR_H

#include <fennec/lang/types.h>

///
///
///
/// \page page_fennec_math_scalar Scalars
/// \anchor scalar
///
/// The \ref fennec Library considers any type that passes ```is_arithmetic<T>``` to be a \ref scalar "Scalar." Bools are
/// supported as a logical type.
///
/// The GLSL Specification, and \ref fennec respectively, defines the following scalar types:
///
/// | Type     | Meaning                      |
/// |----------|------------------------------|
/// |\c bool   | \copybrief fennec::bool_t    |
/// |\c int    | \copybrief fennec::int_t     |
/// |\c uint   | \copybrief fennec::uint_t    |
/// |\c float  | \copybrief fennec::float_t   |
/// |\c double | \copybrief fennec::double_t  |
///
///
///

#include <fennec/math/detail/__fwd.h>
#include <fennec/math/detail/__types.h>

namespace fennec
{

///
/// \brief an unsigned integer
using uint = unsigned int;

}

#endif // FENNEC_MATH_SCALAR_H
